<?php
namespace MyMeals\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Response\Error;
use MyMeals\Services\Auth;
use MyMeals\Services\JWT;
use MyMeals\Util\RequestHeader;

class Login {

    protected $authService;

    /**
     * @param Auth $authService
     */
    public function __construct(Auth $authService, $serverVersion) {
        $this->authService = $authService;
        $this->serverVersion = $serverVersion;
    }

    /**
     * Authenticate and create user session using token
     * @param  DTO\Request\Row $request
     * @param  DTO\Response    $response
     */
    public function login(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $row = $request->getRow();
            $email = $row["email"];
            $password = $row["password"];
            $user = $this->authService->authenticate($email, $password);
            $unit = $this->authService->getUserNmFilial($user);
            $user["NMFILIALSELECIONADA"] = isset($unit["NMFILIAL"]) ? $unit["NMFILIAL"] : "";
            $token = $this->getUserToken($user);
            $this->buildSessionData($user, $response);
        } catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }

    public function testConnection($request, DTO\Response $response) {
        try {
            $response->addDataSet(new DataSet('conn', array('version' => $this->serverVersion)));
        } catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }

    private function buildSessionData($user, $response) {
        $userData = $this->getUserData($user);
        $token = $this->getUserToken($user);
        $userData["session"] = $token;
        $dataSet = new DataSet('user', $userData);
        $response->addDataSet($dataSet);
    }

    /**
     * Retrives a JWT
     * @param  Array $user
     * @return String       JSON Web Token
     */
    private function getUserToken($user) {
        $userData = [
            "cdoperador" => $user["CDOPERADOR"],
            "nmoperador" => $user["NMOPERADOR"],
            "cdfilial" => $user["CDFILIALSELECIONADA"],
            "nmfilial" => $user["NMFILIALSELECIONADA"],
            "nrorg" => $user["NRORG"],
            "dsemailoper" => $user["DSEMAILOPER"]
        ];
        return JWT::generateToken($userData);
    }

     private function getUserData($user) {
        $userData = [
            "cdoperador" => $user["CDOPERADOR"],
            "nmoperador" => $user["NMOPERADOR"],
            "cdfilial" => $user["CDFILIALSELECIONADA"],
            "nmfilial" => $user["NMFILIALSELECIONADA"],
            "nrorg" => $user["NRORG"],
            "dsemailoper" => $user["DSEMAILOPER"]
        ];
        return $userData;
    }

    /**
     * Check session
     * @param  DTO\Request  $request
     * @param  DTO\Response $response
     */
    public function validateSession(DTO\Request $request, DTO\Response $response) {
        try {
            $token = RequestHeader::getKey("session");
            $response->addDataSet(new DataSet("user", JWT::getTokenData($token)));
        } catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }
} 