<?php

namespace MyMeals\Services;

use Zeedhi\Framework\DataSource\FilterCriteria;
use MyMeals\Exceptions;

class Auth {

    const AUTH_TABLE = "operador";
    const EMAIL_COLUMN = "DSEMAILOPER";

    protected $dataSourceManager;

    public function __construct($dataSourceManager) {
        $this->dataSourceManager = $dataSourceManager;
    }

    public function find($filterCriteria) {
        return $this->dataSourceManager->findBy($filterCriteria);
    }

    public function authenticate($email, $password) {
	    $user = $this->getUserByEmail($email);
        
	    if (!$this->isValidUser($user, $password)) {
    		throw new Exceptions\InvalidUserException();
    	}
        return $user;
    }

    public function getUserByEmail($email) {
        $conditions = array();
        $filterCriteria = new FilterCriteria(self::AUTH_TABLE);
        $filterCriteria->addCondition(self::EMAIL_COLUMN, $email);
        $dataSet = $this->find($filterCriteria)->getRows();
    	return isset($dataSet[0]) ? $dataSet[0] : null;
    }

    public function getUserNmFilial($user) {
        $conditions = array(
            array(
                "columnName" => "NRORG",
                "operator" => "=",
                "value" => $user["NRORG"]
            ),
            array(
                "columnName" => "CDFILIAL",
                "operator" => "=",
                "value" => isset($user["CDFILIALSELECIONADA"]) ? $user["CDFILIALSELECIONADA"] : ""
            )
        );
        $filterCriteria = new FilterCriteria("filial", $conditions);
        $dataSet = $this->find($filterCriteria);
        $rows = $dataSet->getRows();
        if (!empty($rows)) {
            return $rows[0];
        } else {
            return array();
        }
    }

    public function isValidUser($user, $password) {
        return $user && $this->isValidPassword($user, $password) && $this->isActiveUser($user);
    }

    private function isActiveUser($user) {
        return $user["IDOPERATIVO"] === "A";
    }

    private function isValidPassword($user, $password) {
    	return $user["CDSENHAOPERWEB"] == md5($password) || $user["CDSENHOPER"] == $this->getCryptoPassword($password);
    }

    private function getCryptoPassword($password) {
        $url = 'http://www.security.teknisa.com/comwebservice/service.asmx?wsdl';
        $options = array(
            'encoding'    => 'ISO-8859-1',
            'trace'       => true,
            'exceptions'  => true,
            'compression' => SOAP_COMPRESSION_ACCEPT | SOAP_COMPRESSION_GZIP,
        );
        $args = array(
            new \SoapVar($password, XSD_STRING, 'string', "http://www.w3.org/2001/XMLSchema")
        );
        $params = array(
            'rdm'    => 'Bib_html.rdmDBGeralHtm',
            'method' => 'GetCriptoPass',
            'args'   => $args,
        );

        try {
            $client = new \SoapClient($url, $options);
            $return = $client->executeCOMMethod($params);
            $cryptoPassword = $return->executeCOMMethodResult;
            return $cryptoPassword;
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage(), $e->getCode());
        }
    }
}