var ZeedhiServices = angular.module("ZeedhiServices", []);

var registerInstance = function(serviceName, instance) {
	ZeedhiServices.service(serviceName, function() {
		return instance;
	});
};

ZeedhiServices.service("requestFactory", ["templateManager", function(templateManager) {
	return new RequestFactory(templateManager);
}]);

ZeedhiServices.service("urlEngine", ["$location", "$rootScope", "eventAggregator", "ScreenService", function($location, $rootScope, eventAggregator, ScreenService) {
	return new UrlEngine($location, $rootScope, eventAggregator, ScreenService);
}]);


ZeedhiServices.service("maskEngine", ["ZhUtil", function(ZhUtil) {
	return new MaskEngine(ZhUtil);
}]);

ZeedhiServices.service("validateEngine", ["ScreenService", "ZhUtil", function(ScreenService, ZhUtil) {
	return new ValidationEngine(ScreenService, ZhUtil);
}]);

ZeedhiServices.service("i18nEngine", ["$injector", function($injector) {
	return new I18nEngine($injector);
}]);

registerInstance("aclEngine", new AclEngine());

registerInstance("chartEngine", new ChartEngine());

ZeedhiServices.service("ZHPromise", ["$q", "ApplicationContext", function($q, ApplicationContext) {
	var zhPromise = new ZHPromise($q);
	ApplicationContext.registerInstance("ZHPromise", zhPromise);
	return zhPromise;
}]);

ZeedhiServices.service("PropertyDefiner", function() {
	function Definer() {

		function accessorDescriptor(field, fun) {
			var desc = {
				enumerable: true,
				configurable: true
			};
			desc[field] = fun;
			return desc;
		}

		this.define = function(object) {

			Object.keys(object).filter(function(key) {
				return typeof object[key] === 'function' && (key.substring(0, 3) === 'get' || key.substring(0, 3) === 'set');
			}).forEach(function(key) {

				var getter = key.substring(3);
				getter = getter[0].toLowerCase() + getter.substring(1);

				if (Object.defineProperty) {
					Object.defineProperty(object, getter, accessorDescriptor(key.substring(0, 3), object[key]));
				} else if (Object.prototype.__defineGetter__) {
					object.__defineGetter__(getter, object[key]);
				} else {
					throw new Error("browser does not support getters");
				}
			});

			return object;
		};

		this.defineProperty = function(object, getter, setter, name) {
			Object.defineProperty(object, name, accessorDescriptor("get", getter));
			Object.defineProperty(object, name, accessorDescriptor("set", setter));
		};

	}
	return new Definer();
});

ZeedhiServices.service("eventHandler", ["restEngine", "requestFactory", "ZhLogger", "validateEngine", "ApplicationContext", function(restEngine, requestFactory, ZhLogger, validateEngine, ApplicationContext) {
	ApplicationContext.registerInstance("ValidationEngine", validateEngine);
	return new EventHandler(restEngine, requestFactory, ZhLogger);
}]);

ZeedhiServices.service("ZhSocketFactory", ["templateManager", "ZhLogger", "ApplicationContext", function(templateManager, ZhLogger, ApplicationContext) {
	var zhSocketFactory = new ZhSocketFactory(ZhLogger, templateManager);
	ApplicationContext.registerInstance("ZhSocketFactory", zhSocketFactory);
	return zhSocketFactory;
}]);

ZeedhiServices.service("templateManager", ["$window", "ZHPromise", "ZhFeatures", "$http", "ApplicationContext", function($window, ZHPromise, ZhFeatures, $http, ApplicationContext) {
	var templateManager = new TemplateManager($window, ZHPromise, ZhFeatures, $http, ApplicationContext);
	ApplicationContext.registerInstance("templateManager", templateManager);
	return templateManager;
}]);

ZeedhiServices.service("ScreenService", ["templateManager", "$rootScope", "i18nEngine", "ApplicationContext", "ZHPromise", function(templateManager, $rootScope, i18nEngine, ApplicationContext, ZHPromise) {
	var instance = new ScreenService(templateManager, $rootScope, i18nEngine, ZHPromise);
	ApplicationContext.registerInstance("ScreenService", instance);
	return instance;
}]);

ZeedhiServices.service('MessageService', ['ScreenService', 'templateManager', '$injector', '$timeout', 'ApplicationContext', function(ScreenService, templateManager, $injector, $timeout, ApplicationContext) {
	var instance = new MessageService(ScreenService, templateManager, $injector, $timeout);
	ApplicationContext.registerInstance('MessageService', instance);
	return instance;
}]);

ZeedhiServices.service("ZhLogger", ["templateManager", "ZHPromise", "ScreenService", function(templateManager, ZHPromise, ScreenService) {
	var loggerInstance = new Logger(Logger.WARNING);

	templateManager.projectPromise.then(function() {
		var project = templateManager.project;

		var logLevel = project.logLevel;

		if (!!logLevel) {
			if (loggerInstance.setLevel(logLevel) === -1) {
				loggerInstance.error("Invalid logLevel assigned to your project: \"" + logLevel + "\".\n" + "Valid ones are: " + Object.keys(Logger).join(
					", ") + ".\nDefaulting to WARNING.");
			}
		} else {
			loggerInstance.warning("Please assign a logLevel property to your project on your containers.json." + "\nDefaulting to WARNING.");
		}

	});

	return loggerInstance;
}]);


ZeedhiServices.service("MethodBuilder", ["ApplicationContext", "$rootScope", function(ApplicationContext, $rootScope) {
	return new MethodBuilder(ApplicationContext, $rootScope);
}]);

ZeedhiServices.service("ZhSprite", ["$http", "$templateCache", "ZHPromise", "ZhLogger", function($http, $templateCache, ZHPromise, ZhLogger) {
	var ZhSprite = function() {
		var self = this;
		var SPRITE = "sprite";
		var SPRITE_URL = "spriteUrl";
		var spriteDeferred = ZHPromise.defer();

		function makeSVG(tag, attrs) {
			var el = document.createElementNS("http://www.w3.org/2000/svg", tag);
			for (var k in attrs)
				el.setAttribute(k, attrs[k]);
			return el;
		}

		function buildPreloadIcon(icon) {
			var svg = makeSVG(icon.svgTag, icon.attrs);
			if (icon.children) {
				icon.children.forEach(function(child) {
					$(svg).append(makeSVG(child.svgTag, child.attrs));
				});
			}
			return svg;
		}

		function preloadIcons() {
			var icons = [{
				id: "border",
				svgTag: "circle",
				attrs: {
					cx: "20",
					cy: "20",
					r: "18"
				}
			}];
			icons.forEach(function(icon) {
				$templateCache.put(icon.id, buildPreloadIcon(icon));
			});
		}

		this.loadSprite = function(spriteUrl) {
			if (!self.getSprite()) {
				$http.get(spriteUrl, {
					cache: true
				}).then(function(response) {
					$templateCache.put(SPRITE, jQuery.parseXML(response.data));
					spriteDeferred.resolve();
				}, function() {
					ZhLogger.error("Custom sprite file could not be loaded.");
				});
			} else {
				spriteDeferred.resolve();
			}
			return spriteDeferred;
		};

		this.getSprite = function() {
			return $templateCache.get(SPRITE);
		};

		this.getIcon = function(iconName) {
			var iconElement = $templateCache.get(iconName);
			if (!iconElement) {
				iconElement = $templateCache.get(SPRITE).querySelector("#" + iconName);
				this.addIcon(iconName, iconElement);
			}
			return iconElement;
		};

		this.addIcon = function(iconName, svg) {
			$templateCache.put(iconName, svg);
		};

		this.setSpriteUrl = function(spriteUrl) {
			$templateCache.put(SPRITE_URL, spriteUrl);
		};

		this.getSpriteUrl = function() {
			return $templateCache.get(SPRITE_URL);
		};

		preloadIcons();
	};

	return new ZhSprite();
}]);

ZeedhiServices.service("utils", function() {

	var Utils = function() {
		this.formatDate = function(date, format) {
			return moment(date).format(format);
		};
	};
	return new Utils();
});

function Memoizer(maxSize) {
	var keys = [];
	var values = [];

	this.add = function(key, value) {
		if (values.length > maxSize) {
			values.shift();
			keys.shift();
		}
		values.push(value);
		keys.push(key);
	};

	this.has = function(key) {
		return keys.indexOf(key) !== -1;
	};

	this.get = function(key) {
		var index = keys.indexOf(key);
		return values[index];
	};
}

ZeedhiServices.service("Memoizer", function() {

	function M() {
		this.create = function(maxSize) {
			return new Memoizer(maxSize);
		};
	}
	return new M();
});

ZeedhiServices.service("eventEngine", ["eventFactory", "eventHandler", "templateManager", "ScreenService", "NavigationLog", function(eventFactory, eventHandler, templateManager, ScreenService, NavigationLog) {
	return new EventEngine(eventFactory, eventHandler, templateManager, ScreenService, NavigationLog);
}]);
/*
 * See eventAggregator to understanding
 */
ZeedhiServices.service("internalEventHandler", ["ZhLogger", function(ZhLogger) {
	return new EventHandler({
		request: function() {
			throw new Error("internalEventHandler shouldn't call method request");
		}
	}, {}, ZhLogger);
}]);
/*
 * See eventAggregator to understanding
 */
ZeedhiServices.service("internalEventEngine", ["eventFactory", "internalEventHandler", "templateManager", "ScreenService", "NavigationLog", function(eventFactory, internalEventHandler, templateManager, ScreenService, NavigationLog) {
	return new EventEngine(eventFactory, internalEventHandler, templateManager, ScreenService, NavigationLog);
}]);

ZeedhiServices.service("eventAggregator", ["internalEventEngine", "ApplicationContext", function(internalEventEngine, ApplicationContext) {
	var instance = new EventAggregator(internalEventEngine);
	ApplicationContext.registerInstance("eventAggregator", instance);
	return instance;
}]);

ZeedhiServices.service("ZhFeatures", ["ApplicationContext", function(ApplicationContext) {
	return new ZhFeatures(ApplicationContext);
}]);

ZeedhiServices.service("requestInterceptingEngine", ["ApplicationContext", "ZHPromise", function(ApplicationContext, ZHPromise) {
	var instance = new RequestInterceptingEngine(ZHPromise);
	ApplicationContext.registerInstance("requestInterceptingEngine", instance);
	return instance;
}]);

ZeedhiServices.service("restEngine", ["ZhLogger", "$http", "templateManager", "ScreenService", "ZhFeatures", "ZHPromise", "eventAggregator", "ApplicationContext", "requestInterceptingEngine", function(ZhLogger, $http, templateManager, ScreenService, ZhFeatures, ZHPromise, eventAggregator,
	ApplicationContext, requestInterceptingEngine) {
	var instance = new RestEngine(ZhLogger, $http, templateManager, ScreenService, ZhFeatures, ZHPromise, eventAggregator, requestInterceptingEngine);
	ApplicationContext.registerInstance("RestEngine", instance);
	return instance;
}]);

ZeedhiServices.service("requestEngine", ["restEngine", function(restEngine) {
	return new RequestEngine(restEngine);
}]);

ZeedhiServices.service("requestDataSourceEngine", ["requestEngine", "requestFactory", "MethodBuilder", "ScreenService", "ZHPromise", "ZhUtil", "dataSourceHolder", "templateManager", function(requestEngine, requestFactory, MethodBuilder, ScreenService, ZHPromise, ZhUtil, dataSourceHolder, templateManager) {
	return new RequestDataSourceEngine(requestEngine, requestFactory, MethodBuilder, ScreenService, ZHPromise, ZhUtil, dataSourceHolder, templateManager);
}]);

ZeedhiServices.service("localStorageEngine", ["ZhLogger", "requestDataSourceEngine", "requestFactory", "requestEngine", "templateManager", "dataSourceHolder", "ZHPromise", "ZhUtil", "Query", function(ZhLogger, requestDataSourceEngine, requestFactory, requestEngine, templateManager, dataSourceHolder, ZHPromise, ZhUtil, Query) {
	return new LocalStorageEngine(ZhLogger, requestDataSourceEngine, requestFactory, requestEngine, templateManager, dataSourceHolder, ZHPromise,
		ZhUtil, Query);
}]);

ZeedhiServices.service("memoryStorageEngine", ["ZhLogger", "requestDataSourceEngine", "requestFactory", "requestEngine", "templateManager", "dataSourceHolder", "ZHPromise", "ZhUtil", "Query", function(ZhLogger, requestDataSourceEngine, requestFactory, requestEngine, templateManager, dataSourceHolder, ZHPromise, ZhUtil, Query) {
	return new MemoryStorageEngine(ZhLogger, requestDataSourceEngine, requestFactory, requestEngine, templateManager, dataSourceHolder, ZHPromise,
		ZhUtil, Query);
}]);

ZeedhiServices.service("SQLStorageStrategy", ["ZhLogger", "requestDataSourceEngine", "ZHPromise", function(ZhLogger, requestDataSourceEngine, ZHPromise) {
	return new ZhSQLStorageStrategy(ZhLogger, requestDataSourceEngine, ZHPromise);
}]);

ZeedhiServices.service("metaDataFactory", ["localStorageEngine", "requestDataSourceEngine", "SQLStorageStrategy", "restEngine", "eventEngine", "ScreenService", "aclEngine", "templateManager", "ZhUtil", "ZHPromise", "dataSourceHolder", "ZhLogger", "DecodeValueFieldService", "ZhFeatures", "memoryStorageEngine", "validateEngine", "MessageService", function(localStorageEngine, requestDataSourceEngine, SQLStorageStrategy, restEngine, eventEngine,
	ScreenService, aclEngine, templateManager, ZhUtil, ZHPromise, dataSourceHolder, ZhLogger, DecodeValueFieldService, ZhFeatures, memoryStorageEngine, validateEngine, MessageService) {
	return new MetaDataFactory(localStorageEngine, requestDataSourceEngine, SQLStorageStrategy, restEngine, eventEngine, ScreenService, aclEngine,
		templateManager, ZhUtil, ZHPromise, dataSourceHolder, ZhLogger, DecodeValueFieldService, ZhFeatures, memoryStorageEngine, validateEngine, MessageService);
}]);

ZeedhiServices.service("renderEngine", ["ZhLogger", "metaDataFactory", "templateManager", "$timeout", "ZHPromise", "restEngine", "MetaDataLoader", "eventAggregator", function(ZhLogger, metaDataFactory, templateManager, $timeout, ZHPromise, restEngine, MetaDataLoader, eventAggregator) {
	return new RenderEngine(ZhLogger, metaDataFactory, templateManager, $timeout, ZHPromise, restEngine, MetaDataLoader, eventAggregator);
}]);

ZeedhiServices.service("wizardEngine", ["renderEngine", "templateManager", "metaDataFactory", "eventFactory", "eventHandler", "eventEngine", "MetaDataLoader", function(renderEngine, templateManager, metaDataFactory, eventFactory, eventHandler, eventEngine, MetaDataLoader) {
	return new WizardEngine(renderEngine, templateManager, metaDataFactory, eventFactory, eventHandler, eventEngine, MetaDataLoader);
}]);

ZeedhiServices.service("dataSourceHolder", ["restEngine", "ZHPromise", "ZhLogger", function(restEngine, ZHPromise, ZhLogger) {
	return new DataSourceHolder(ZHPromise, restEngine, ZhLogger);
}]);

ZeedhiServices.service("ApplicationContext", function() {
	var ApplicationContext = {
		instanceKeys: [],
		instances: [],
		getArgsToFunction: function() {
			return {
				args: this.instances,
				keys: this.instanceKeys
			};
		},
		registerInstance: function(name, instance) {
			if (!ApplicationContext[name]) {
				if (!!instance) {
					ApplicationContext[name] = instance;
					ApplicationContext.instanceKeys.push(name);
					ApplicationContext.instances.push(instance);
				}
			} else {
				throw new Error("[" + name + "] have already been registered!");
			}
		},
		removeInstance: function(name) {
			if (ApplicationContext.hasOwnProperty(name)) {
				var instance = ApplicationContext[name];
				delete ApplicationContext[name];
				var index = ApplicationContext.instanceKeys.indexOf(name);
				if (index > -1) {
					ApplicationContext.instanceKeys.splice(index, 1);
				}
				index = ApplicationContext.instances.indexOf(instance);
				if (index > -1) {
					ApplicationContext.instances.splice(index, 1);
				}
			}
		}
	};
	return ApplicationContext;
});

ZeedhiServices.service("eventFactory", ["MethodBuilder", "ScreenService", function(MethodBuilder, ScreenService) {
	return new EventFactory(MethodBuilder, ScreenService);
}]);

ZeedhiServices.service("ContextRegister", ["ApplicationContext", "$injector", function(ApplicationContext, $injector) {
	function ContextRegister() {
		function instantiate(service) {
			return $injector.instantiate(service, ApplicationContext);
		}
		this.register = function(name, service) {
			var serviceInstance = typeof(service) !== "function" ? service : instantiate(service);
			ApplicationContext.registerInstance(name, serviceInstance);
		};
	}
	return new ContextRegister();
}]);

ZeedhiServices.service("RepositoryFactory", ["SQLStorageStrategy", "localStorageEngine", "requestDataSourceEngine", "templateManager", "dataSourceHolder", "ZhLogger", "eventEngine", "IndexedDBEngine", "Query", "memoryStorageEngine", function(SQLStorageStrategy, localStorageEngine, requestDataSourceEngine, templateManager, dataSourceHolder, ZhLogger, eventEngine, IndexedDBEngine, Query, memoryStorageEngine) {
	return new RepositoryFactory(SQLStorageStrategy, localStorageEngine, requestDataSourceEngine, templateManager, dataSourceHolder, ZhLogger,
		eventEngine, IndexedDBEngine, Query, memoryStorageEngine);
}]);

ZeedhiServices.service("Query", function() {
	return new QueryBuilder();
});

ZeedhiServices.service("FactoryRegister", ["ApplicationContext", "$injector", function(ApplicationContext, $injector) {
	function FactoryRegister() {
		function instantiate(service) {
			return $injector.instantiate(service, ApplicationContext);
		}

		var factories = [];

		this.register = function(entity, name, factory) {

			if (factory.name === "") {
				throw new Error("FactoryRegister: The factory registered with the name of '" + name + "' cannot be an anonymous function.");
			} else if (entity.name === "") {
				throw new Error("FactoryRegister: The entity produced by the " + factory.name + " cannot be an anonymous function.");
			}

			factory.prototype.ProducedClass = entity;
			var instantiated = instantiate(factory);
			ApplicationContext[name] = instantiated;
			factories[entity.name] = instantiated;
		};

		this.getAllFactories = function() {
			return factories;
		};
	}
	return new FactoryRegister();
}]);

ZeedhiServices.service("IndexedDBEngine", ["dataSourceHolder", "requestDataSourceEngine", "requestEngine", "requestFactory", "templateManager", "ZHPromise", "ZhLogger", "ZhUtil", "Query", function(dataSourceHolder, requestDataSourceEngine, requestEngine, requestFactory, templateManager,
	ZHPromise, ZhLogger, ZhUtil, Query) {
	return new IndexedDBEngine(dataSourceHolder, requestDataSourceEngine, requestEngine, requestFactory, templateManager, ZHPromise, ZhLogger, ZhUtil, Query);
}]);

ZeedhiServices.service("ZhUtil", ["ApplicationContext", "Query", "$filter", "dataSourceHolder", "ZhLogger", function(ApplicationContext, Query, $filter, dataSourceHolder, ZhLogger) {
	return new ZhUtil(ApplicationContext, Query, $filter, dataSourceHolder, ZhLogger);
}]);

ZeedhiServices.service("DecodeValueFieldService", ["templateManager", "ScreenService", "maskEngine", "ZhUtil", function(templateManager, ScreenService, maskEngine, ZhUtil) {
	return new DecodeValueFieldService(templateManager, ScreenService, maskEngine, ZhUtil);
}]);

ZeedhiServices.service("decodeTemplateFieldService", ["templateManager", function(templateManager) {
	return new DecodeTemplateFieldService(templateManager);
}]);

ZeedhiServices.service("SelectService", ["Query", function(Query) {
	return new SelectService(Query);
}]);

ZeedhiServices.service("CrosstabService", ["maskEngine", "ZhUtil", function(maskEngine, ZhUtil) {
	return new CrosstabService(maskEngine, ZhUtil);
}]);

ZeedhiServices.service("ZhLocation", ["ZHPromise", "ScreenService", function(ZHPromise, ScreenService) {
	return new ZhLocation(ZHPromise, ScreenService);
}]);

ZeedhiServices.service("ZhKeyboardService", ["eventEngine", function(eventEngine) {
	return new ZhKeyboardService(eventEngine);
}]);

ZeedhiServices.service("SynchronizableStorageEngineFactory", ["ZHPromise", function(ZHPromise) {
	return new SynchronizableStorageEngineFactory(ZHPromise);
}]);

ZeedhiServices.service("PreferenceManager", ["SynchronizableStorageEngineFactory", "localStorageEngine", "requestDataSourceEngine", "Query", "ScreenService", "ZHPromise", "ZhUtil", "RepositoryFactory", function(SynchronizableStorageEngineFactory, localStorageEngine, requestDataSourceEngine, Query, ScreenService, ZHPromise, ZhUtil, RepositoryFactory) {
	return new PreferenceManager(SynchronizableStorageEngineFactory, {}, localStorageEngine, requestDataSourceEngine, Query, ScreenService, ZHPromise, ZhUtil, RepositoryFactory);
}]);

ZeedhiServices.service("ContainerRepository", ["PreferenceManager", function(PreferenceManager) {
	return new ContainerRepository(PreferenceManager);
}]);

ZeedhiServices.service("ContainerDecoratorPreference", ["ContainerRepository", "ScreenService", "ZHPromise", "containerMerger", function(ContainerRepository, ScreenService, ZHPromise, containerMerger) {
	return new ContainerDecoratorPreference(ContainerRepository, ScreenService, ZHPromise, containerMerger);
}]);

ZeedhiServices.service("ContainerDecoratorProvider", ["ContainerDecoratorPreference", function(ContainerDecoratorPreference) {
	return new ContainerDecoratorProvider(ContainerDecoratorPreference);
}]);

ZeedhiServices.service("JSONContainerLoader", ["restEngine", "ZHPromise", "ScreenService", function(restEngine, ZHPromise, ScreenService) {
	return new JSONContainerLoader(restEngine, ZHPromise, ScreenService);
}]);

ZeedhiServices.service("ContainerLoader", ["JSONContainerLoader", "ContainerDecoratorProvider", "ZHPromise", "ScreenService", function(JSONContainerLoader, ContainerDecoratorProvider, ZHPromise, ScreenService) {
	return new ContainerLoader(JSONContainerLoader, ContainerDecoratorProvider, ZHPromise, ScreenService);
}]);

ZeedhiServices.service("MetaDataLoader", ["ContainerLoader", "metaDataFactory", "ScreenService", "ZHPromise",
	"templateManager", "restEngine", "ApplicationContext", "$rootScope", "eventAggregator", "urlEngine",
	function(ContainerLoader, metaDataFactory, ScreenService, ZHPromise, templateManager, restEngine,
		ApplicationContext, $rootScope, eventAggregator, urlEngine) {

		return new MetaDataLoader(ContainerLoader, metaDataFactory, ScreenService, ZHPromise, templateManager,
			restEngine, ApplicationContext, $rootScope, eventAggregator, urlEngine);
	}
]);


ZeedhiServices.service("containerMerger", function() {
	return new ContainerMerger();
});

ZeedhiServices.service("EventChildPropagation", function() {
	return new EventChildPropagation();
});

ZeedhiServices.service("NavigationLog", ["templateManager", "ApplicationContext", "ZhLogger", function(templateManager, ApplicationContext, ZhLogger) {
	return new NavigationLog(templateManager, ApplicationContext, ZhLogger);
}]);

ZeedhiServices.run(function(ZhLogger, ScreenService, templateManager) {
	ErrorReporting(ZhLogger, ScreenService, templateManager, window);
});

window.Configuration = function(config) {
	ZeedhiServices.run(config);
};