describe("Field", function() {
	var localStorageEngine, requestDataSourceEngine, SQLStorageStrategy, restEngine = null;
	var widget = null;
	var field  = null;
	/*jslint evil: true */
	var eventEngine = new EventEngine(new EventFactory(new MethodBuilder({}, {
		$eval: function(a, b) {
			return eval(a, b);
		}
	}), {}), new EventHandler({}, {}, console), {
		updateTemplate: function() {}
	}, {
		i18n : function(text) {
			return text;
		}
	});
	var aclEngine = new AclEngine();
	var screenService = {
		isPopupOpen: function() {
			return false;
		}
	};
	var metaDataFactory = new MetaDataFactory(localStorageEngine, requestDataSourceEngine,
		SQLStorageStrategy, restEngine, eventEngine, screenService, aclEngine, {
			project: {}, updateTemplate: function () {}
		}, {}, new ZHPromise(Q), { getDataSourceLazy: function() {} }, {}, {}, {}, {}, {}, {});
	var baseContainer = function() {
		return {
			"widgets": [{
				"events": [],
				"dataSource": {
					"name": "/test",
					"data": [{}, {}, {}, {}]
				},
				"fields": [				{
					"name": "image",
					"source": "images/zeedhi-demo.svg",
					"size": 10,
					"class": "10 col-xs-offset-1",
					"isVisible": true,
					"template": "field/image-static.html",
					"dataSource" : {},
					"mask": {
						"type": "zerofill"
					}
				}],
				"active": true,
				"actions": []
			}]
		};
	};
	
	beforeEach(function() {
		container = metaDataFactory.containerFactory(baseContainer());
		widget = container.widgets[0];
		field = container.widgets[0].fields[0];
	});
	
	it("readOnly by default is false", function() {
		expect(field.readOnly).toEqual(false);
	});
	
	it("aclReadOnly by default is false", function() {
		expect(field.aclReadOnly).toEqual(false);
	});
	
	it("readOnlyOnEdit by default is false", function() {
		expect(field.readOnlyOnEdit).toEqual(false);
	});
	
	it("isReadOnly() by default is false", function() {
		expect(field.isReadOnly()).toEqual(false);
	});
	
	it("isReadOnly() is false if readOnly:false, aclReadOnly:flase, readOnlyOnEdit: true and widget.currentRow.__new_row = true", function() {
		field.readOnly = false;
		field.aclReadOnly = false;
		field.readOnlyOnEdit = true;
		field.widget.currentRow = { __is_new : true};
		expect(field.isReadOnly()).toEqual(false);
	});
	
	it("isReadOnly() is true if readOnly:false, aclReadOnly:flase, readOnlyOnEdit: true and widget.currentRow.__new_row = false", function() {
		field.readOnly = false;
		field.aclReadOnly = false;
		field.readOnlyOnEdit = true;
		field.widget.currentRow = { __is_new : false};
		expect(field.isReadOnly()).toEqual(true);
	});

	it("isReadOnly() is true if readOnly:true, aclReadOnly:flase, readOnlyOnEdit: true and widget.currentRow.__new_row = true", function() {
		field.readOnly = true;
		field.aclReadOnly = false;
		field.readOnlyOnEdit = true;
		field.widget.currentRow = { __is_new : false};
		expect(field.isReadOnly()).toEqual(true);
	});

	it("isReadOnly() is true if readOnly:false, aclReadOnly:true, readOnlyOnEdit: true and widget.currentRow.__new_row = true", function() {
		field.readOnly = false;
		field.aclReadOnly = true;
		field.readOnlyOnEdit = true;
		field.widget.currentRow = { __is_new : false};
		expect(field.isReadOnly()).toEqual(true);
	});

	it("isReadOnly() is false if readOnly:false, aclReadOnly:false, readOnlyOnEdit: false and widget.currentRow.__new_row = false", function() {
		field.readOnly = false;
		field.aclReadOnly = false;
		field.readOnlyOnEdit = false;
		field.widget.currentRow = { __is_new : false};
		expect(field.isReadOnly()).toEqual(false);
	});
	
	it("isReadOnly() is false if readOnly:false, aclReadOnly:false, readOnlyOnEdit: false and widget.currentRow = undefined", function() {
		field.readOnly = false;
		field.aclReadOnly = false;
		field.readOnlyOnEdit = false;
		field.widget.currentRow = undefined;
		expect(field.isReadOnly()).toEqual(false);
	});
	
	it("isReadOnly() is false if readOnly:false, aclReadOnly:false, readOnlyOnEdit: false and widget.currentRow = null", function() {
		field.readOnly = false;
		field.aclReadOnly = false;
		field.readOnlyOnEdit = false;
		field.widget.currentRow = null;
		expect(field.isReadOnly()).toEqual(false);
	});

	it("isReadOnly() is true if readOnly:true, aclReadOnly:false, readOnlyOnEdit: false and widget.currentRow = undefined", function() {
		field.readOnly = true;
		field.aclReadOnly = false;
		field.readOnlyOnEdit = false;
		field.widget.currentRow = undefined;
		expect(field.isReadOnly()).toEqual(true);
	});
	
	it("isReadOnly() is true if readOnly:false, aclReadOnly:true, readOnlyOnEdit: false and widget.currentRow = null", function() {
		field.readOnly = false;
		field.aclReadOnly = true;
		field.readOnlyOnEdit = true;
		field.widget.currentRow = null;
		expect(field.isReadOnly()).toEqual(true);
	});
	
	it("isReadOnly() is true if readOnly:false, aclReadOnly:false, readOnlyOnEdit: false and widget.currentRow.__new_row = undefined", function() {
		field.readOnly = false;
		field.aclReadOnly = false;
		field.readOnlyOnEdit = true;
		field.widget.currentRow = {};
		expect(field.isReadOnly()).toEqual(true);
	});
	
	it("reload() should return a promise", function() {
		field.dataSource.load = function() {
			return {
				then: function(fullfill) {
					if (fullfill) {
						fullfill();
					}
				}};
		};
		
		var mock = ZhHelper.buildMock();
		
		field.reload([]).then(function () {
			mock.whenSuccess();
		});
		
		expect(mock.whenSuccess).toHaveBeenCalled();
	});
	
	it("reload() should return a promise even if the field does not have a dataSource", function() {
		field.dataSource = null;
		
		var mock = ZhHelper.buildMock();
		
		field.reload([]).then(function () {
			mock.whenSuccess();
		});
		
		expect(mock.whenSuccess).toHaveBeenCalled();
	});
	
	it("getparent() should return the field widget", function () {
		field.widget = "justChecking";
		
		expect(field.getParent()).toEqual("justChecking");
	});
	
	it("value() should return the widget currentRow value for the field name if no parameter is passed", function () {
		field.widget.newRow();
		field.widget.currentRow[field.name] = "myVal";
		
		expect(field.value()).toEqual("myVal");
	});
	
	it("value() should set its widget currentRow value for the field if a parameter is passed", function () {
		field.widget.newRow();
		
		field.value("myVal");
		expect(field.widget.currentRow[field.name]).toEqual("myVal");
	});
	
	it("setValue() should set the widget ", function () {
		field.widget.newRow();
		
		field.value("myVal");
		expect(field.widget.currentRow[field.name]).toEqual("myVal");
	});
	
	it("clearValue() should clear the widget currentRow value for the field", function () {
		field.widget.newRow();
		field.widget.currentRow[field.name] = "oldVal";
		
		expect(field.value()).toEqual("oldVal");
		field.clearValue();
		
		expect(field.value()).toEqual("");
	});
	
	//TODO: Check if this is right (WTF field with currentRow?)
	it("setCurrentRow() should should set the currentRow", function () {
		field.beforeMoveRow = function () {};
		field.afterMoveRow = function() {};
		
		var row = {firstVal: "wow", secondVal: "WOWOWOOW"};
		field.setCurrentRow(row);
		
		expect(field.currentRow).toBe(row);
	});
});