describe("SelectMonthController", function () {
	var selectMonthController;
	var $scope;
	var $rootScope;
	var container;
	var widget;
	var field;
	
	var baseContainer = ZhHelper.baseContainerWithTwoWidgets;
	var baseFields = function () {
		return [
			{
				name : "name",
				isVisible: true,
				aclIsVisible: true,
				showOnList: true,
				skipSearch: false,
			},
			{
				name: "birth",
				isVisible: true,
				aclIsVisible: true,
				showOnList: true,
				skipSearch: false,
			}
		];
	};
	
	
	beforeEach(function () {
		module(function ($provide) {
			//Providing a new $timeout because $timeout.flush is not flushing pending $timeout calls
			$provide.constant("$timeout", setTimeout);
			$provide.constant("$q", Q);
		});
		module("ZeedhiApp");
		module("ZeedhiServices");
		
		inject(function (_$rootScope_, $controller, metaDataFactory, ScreenService) {
			screenService = ScreenService;
			container = baseContainer();
			container.widgets[0].template = "widget/form.html";
			container.widgets[0].fields = baseFields();
			container.widgets[0].fields[0].template = "field/select-month.html";
			container = metaDataFactory.containerFactory(container);
			widget = container.widgets[0];
			$rootScope = _$rootScope_;
			
			$scope = _$rootScope_.$new();
			$scope.$parent = _$rootScope_;
			$scope.widget = widget;
			$scope.field = widget.fields[0];
			field = $scope.field;
			
			selectMonthController = $controller("SelectMonthController", {
				$scope: $scope,
				$rootScope: $rootScope
			});
		});
	});
	
	it("$scope.openField: should do nothing if the field is readOnly", function () {
		field.readOnly = true;
		spyOn(screenService, "openPopup").and.callThrough();
		$scope.openField(field);
		expect(screenService.openPopup).not.toHaveBeenCalled();
		expect(selectMonthController.getSelectWidget()).toBeUndefined();
	});
	
	it("$scope.openField: should create a widget and open it as a popup", function () {
		spyOn(screenService, "openPopup").and.callThrough();
		$scope.openField(field);
		expect(screenService.openPopup).toHaveBeenCalled();
		expect(selectMonthController.getSelectWidget()).not.toBeUndefined();
	});
	
	it("selectWidget.setSelected: should set the widget currentRow and call field's change event", function () {
		field.change = function() {};
		spyOn(field, "change").and.callThrough();
		$scope.openField(field);
		var widget = selectMonthController.getSelectWidget();
		widget.setSelected({ owner: { selectField: field }, row: {} });
		expect(field.change).toHaveBeenCalled();
		expect(field.widget.currentRow[field.name]).toEqual({});
	});
	
	it("$scope.clearField: should clear the widget currentRow value and set the field currentYear to the current", function () {
		field.change = function() {};
		spyOn(field, "change").and.callThrough();
		$scope.clearField(field);
		
		expect(field.change).toHaveBeenCalled();
		expect(field.widget.currentRow[field.name]).toEqual("");
		expect(field.currentYear).toEqual(moment().get("year"));
	});

	it("watch the widget currentRow for the field: should set the currentROw value empty if there is no value", function () {
		field.widget.currentRow[field.name] = "";
		$scope.$apply();
		expect(field.widget.currentRow[field.name]).toEqual("");
		
		field.minDate = null;
		field.maxDate = null;
		field.widget.currentRow[field.name] = "01/2016";
		$scope.$apply();
		expect(field.formattedValue).toEqual("01/2016");
	});
	
	it("watch the widget currentRow for the field: should set the formatted value of the currentRow value", function () {
		field.widget.currentRow[field.name] = "05/2016";
		field.minDate = "01/2016";
		field.maxDate = "06/2016";
		field.dateFormat = "MM/YYYY";
		$scope.$apply();
		expect(field.formattedValue).toEqual("05/2016");
		
		field.widget.currentRow[field.name] = "06/2016";
		field.minDate = null;
		field.maxDate = "07/2016";
		$scope.$apply();
		expect(field.formattedValue).toEqual("06/2016");
		
		field.widget.currentRow[field.name] = "05/2016";
		field.minDate = "01/2016";
		field.maxDate = null;
		$scope.$apply();
		expect(field.formattedValue).toEqual("05/2016");
	});
});