describe('zhAcl', function() {

	var $compile;
	var scope;
	var templateManager;
	var menus;

	beforeEach(function() {
		module('ZeedhiServices');
		module('ZeedhiDirectives');
		inject(function(_$compile_, _$rootScope_, _templateManager_) {
			$compile = _$compile_;
			scope = _$rootScope_.$new();
			templateManager = _templateManager_;
		});

		templateManager.userId = "0001";
		templateManager.aclRules = {
			"1984920": {
				"type": "menu",
				"itemName": "menu1",
				"userGroups": {
					"0001": {
						"rule": "I"
					}
				}
			},
			"100347549758": {
				"type": "container",
				"itemName": "container1",
				"userGroups": {
					"0001": {
						"rule": "I"
					}
				}
			},
			"548115975": {
				"itemName": "widget1",
				"type": "widget",
				"userGroups": {
					"0001": {
						"rule": "A"
					}
				}
			},
			"947582": {
				"type": "action",
				"itemName": "save",
				"userGroups": {
					"0001": {
						"rule": "I"
					}
				}
			}
		};
		templateManager.project = {
			"name": "project 1",
			"groupMenu": [
				{
					"id": 150241018,
					"name": "parentMenu1",
					"label": "parentMenu1",
					"hasChild": true,
					"menus": [
						{
							"id": 1984920,
							"label": "menu1",
							"name": "menu1",
							"windowName": "container4",
							"icon": "home icon-white"
						}, {
							"id": 1248532,
							"label": "menu4",
							"name": "menu4",
							"windowName": "container3",
							"icon": "home icon-white"
						}
					]
				}
			]
		};
		templateManager.container = {
			"id": 100347549758,
			"name": "container1",
			"isVisible": true,
			"widgets": [
				{
					"id": 548115975,
					"name": "widget1",
					"isVisible": true,
					"actions": [
						{
							"id": 947582,
							"name": "save",
							"isVisible": true
						}
					],
					"fields": [
						{
							"id": 2548552,
							"name": "field1"
						}
					]
				}
			]
		};

		menus = templateManager.project.groupMenu;
	});

	it('should not apply acl when project acl is disable', function() {
		templateManager.project.aclEnable = false;
		$compile('<input type="text" ng-model="value" zh-acl></input>')(scope);
		scope.$digest();
		expect(menus[0].aclIsVisible).toBeUndefined();
		expect(menus[0].aclReadOnly).toBeUndefined();

		expect(templateManager.container.aclIsVisible).toBeUndefined();
		expect(templateManager.container.aclReadOnly).toBeUndefined();
	});

	it('should apply acl when project acl is enable', function() {
		templateManager.project.aclEnable = true;
		templateManager.project.aclDefaultRule = "A";
		$compile('<input type="text" ng-model="value" zh-acl></input>')(scope);
		scope.$digest();
		expect(menus[0].aclIsVisible).toBe(true);
		expect(menus[0].aclReadOnly).toBe(false);

		expect(menus[0].menus[0].aclIsVisible).toBe(false);
		expect(menus[0].menus[0].aclReadOnly).toBe(false);

		expect(templateManager.container.aclIsVisible).toBe(true);
		expect(templateManager.container.aclReadOnly).toBe(false);

		expect(templateManager.container.widgets[0].aclIsVisible).toBe(true);
		expect(templateManager.container.widgets[0].aclReadOnly).toBe(false);

		expect(templateManager.container.widgets[0].actions[0].aclIsVisible).toBe(false);
		expect(templateManager.container.widgets[0].actions[0].aclReadOnly).toBe(false);
	});

});