function FilterWidgetsCreator(metaDataFactory, ScreenService) {

	this.createFiltersWidget = function (gridWidget, functionsToInject){
		var widget = metaDataFactory.widgetFactory({
			"id": 1,
			"showAsModal": true,
			"container": gridWidget.container,
			"name": "popUpFiltersList",
			"label": ScreenService.i18n("Filters"),
			"isVisible": true,
			"template": "widget/grid.html",
			"showCheckbox": true,
			"actions": actionsFiltersListWidget,
			"styles": [],
			"floatingControl": {
				"refreshAction": false,
				"customizationAction": {
					"layout": false,
					"filter": false,
					"view": false
				},
				"actions": []
			},
			"events": [
				{
					"id": 2,
					"name": "WidgetOnEnter",
					"code": "args.owner.fillFilterList(args.owner);"
				},
				{
					"id": 3,
					"name": "WidgetAfterMoveRow",
					"code": "args.owner.openEditFilter(args.owner, args.row);"
				}
			],
			"dataSource": {
				"localStorage": false,
				"rest": false,
				"data": []
			},
			"fields": [
				{
					"id": 1,
					"name": "label",
					"label": ScreenService.i18n("Filter name"),
					"showOnList": true,
					"template": "field/text-edit.html"
				},
				{
					"id": 2,
					"name": "filterTypeLabel",
					"label": ScreenService.i18n("Type"),
					"showOnList": true,
					"template": "field/text-edit.html"
				},
				{
					"id": 4,
					"name": "defaultPreference",
					"isVisible": false,
					"showOnList": true,
					"label": ScreenService.i18n("Default"),
					"template": "field/checkbox.html",
					"align" : "center",
					"valueToTrue": true,
					"valueToFalse": false,
					"inlineEditable" : "checkbox",
					"hideCheckAllButton": true,
					"events": [{
						"id": 1,
						"name": "FieldOnChange",
						"code": "args.owner.field.widget.setAsDefaultPreference(args.row, args.owner.field.widget);"
					}]
				}
			]
		});
		return Util.extend(widget, functionsToInject);
	};

	var actionsFiltersListWidget = [
		{
			"id": 2,
			"name": "NEW",
			"isVisible": true,
			"showAsAction": "always",
			"icon": "plus icon-white",
			"tooltip": ScreenService.i18n("New"),
			"events": [{
				"id": 1,
				"name": "ActionEvent",
				"code": "args.owner.widget.openCreateNewFilterWidget(args.owner.widget)"
			}]
		},
		{
			"id": 3,
			"name": "cancelAction",
			"label": ScreenService.i18n("Close"),
			"showAsAction": "back",
			"hideIcon": true,
			"isVisible": true,
			"events": [{
				"id": 1,
				"name": "ActionEvent",
				"code": "args.owner.widget.closeFilterPopup()"
			}]
		},
		{
			"id": 4,
			"name": "deletePreference",
			"isVisible": true,
			"icon": "trash",
			"showAsAction": "checked_rows",
			"condition": "row.acl_edit",
			"tooltip": ScreenService.i18n("Delete preference"),
			"events": [
				{
					"id": 1,
					"name": "ActionEvent",
					"code": "args.owner.widget.deletePreferenceClick(args.owner.widget);"
				}
			]
		}
	];

	this.createNewFilterWidget = function(gridWidget, functionsToInject) {
		var widget = metaDataFactory.widgetFactory({
			"id": "335689",
			"isVisible": true,
			"name": "saveFilterPopup",
			"showAsModal": true,
			"container": gridWidget.container,
			"label": ScreenService.i18n("Create a new filter"),
			"template": "widget/form.html",
			"currentRow": {},
			"descriptionOrder": false,
			"fields": [
				{
					"id": 1,
					"label": ScreenService.i18n("Name"),
					"name": "name",
					"showOnList": true,
					"class": "6",
					"showOnForm": true,
					"events": [],
					"template": "field/text-edit.html",
					"validations": {
						"required": {
							"message": ScreenService.i18n("Required Field")
						}
					}
				},
				{
					"name": "type",
					"label": ScreenService.i18n("Type"),
					"class": "6",
					"isVisible": true,
					"readOnly": false,
					"showOnList": true,
					"showOnForm": true,
					"showOnView": true,
					"template": "field/select-native.html",
					"valueField": "value",
					"descriptionField": "label",
					"events": [],
					"hideEmptyValue": false,
					"dataSource": {
						"data": []
					},
					"validations": {
						"required": {
							"message": ScreenService.i18n("Required Field")
						}
					}
				}
			],
			"actions": [{
					"id": 2,
					"name": "cancelAction",
					"label": ScreenService.i18n("Cancel"),
					"showAsAction": "back",
					"hideIcon": true,
					"isVisible": true,
					"events": [{
						"id": 1,
						"name": "ActionEvent",
						"code": "ScreenService.closePopup();"
				}]
				}, {
					"id": 2,
					"name": "saveFilterAction",
					"label": ScreenService.i18n("Continue"),
					"showAsAction": "never",
					"hideIcon": true,
					"isVisible": true,
					"events": [{
						"id": 1,
						"name": "ActionEvent",
						"code": "args.owner.widget.createNewFilter(args.owner.widget)"
					}]
				}
			]
		});
		return Util.extend(widget, functionsToInject);
	};

	this.createChangeFilterTypeWidget = function(gridWidget, functionsToInject){
		var widget = metaDataFactory.widgetFactory({
			"id": 8998,
			"name": "changePreferenceTypeWidget",
			"showAsModal": true,
			"container": gridWidget.container,
			"label": ScreenService.i18n("Change filter type"),
			"template": "widget/form.html",
			"showDescriptionOrder": false,
			"isVisible": true,
			"fields": [
				{
					"name": "filterType",
					"label": ScreenService.i18n("New filter type"),
					"class": "12",
					"isVisible": true,
					"readOnly": false,
					"showOnList": true,
					"showOnForm": true,
					"showOnView": true,
					"template": "field/select-native.html",
					"valueField": "value",
					"descriptionField": "label",
					"events": [],
					"hideEmptyValue": false,
					"dataSource": {
						"data":[]
					},
					"validations": {
						"required":{
							"message": ScreenService.i18n("Required Field")
						}
					}
				}
			],
			"events": [],
			"actions":[
				{
					"id": 1,
					"name": "canceladd",
					"label": ScreenService.i18n("Cancel"),
					"isVisible": true,
					"hideIcon": true,
					"showAsAction": "back",
					"events": [{
						"id": 1,
						"name": "ActionEvent",
						"code": "ScreenService.closePopup();"
					}]
				},
				{
					"id": 1,
					"name": "tranferFilter",
					"label": ScreenService.i18n("Save"),
					"isVisible": true,
					"showAsAction": "never",
					"events": [
						{
							"id": 1,
							"name": "ActionEvent",
							"code": "args.owner.widget.saveTransferFilterClick(args.owner.widget)"
						}
					]
				}
			]
		});
		return Util.extend(widget, functionsToInject);
	};

	var actionsPropertiesWidget = [
		{
			"label": ScreenService.i18n("Cancel"),
			"showAsAction": "back",
			"isVisible": true,
			"showOnForm": true,
			"showOnList": true,
			"hideIcon": true,
			"events": [{
				"id": 1,
				"name": "ActionEvent",
				"code": "args.owner.widget.backToPreviousWidget(args.owner.widget)"
			}]
		}, {
			"label": ScreenService.i18n("OK"),
			"showAsAction": "never",
			"isVisible": true,
			"showOnForm": true,
			"showOnList": true,
			"events": [{
				"id": 1,
				"name": "ActionEvent",
				"code": "args.owner.widget.saveFieldChanges(args.owner.widget, args.row)"
			}]
		}
	];

	this.createFormFilterWidget = function(gridWidget, functionsToInject) {
		var widget = metaDataFactory.widgetFactory({
			"id": "31231231231",
			"isVisible": true,
			"name": "filterPopup",
			"orderFieldsBySequenceInForm": true,
			"container": gridWidget.container,
			"label": ScreenService.i18n("Filter"),
			"template": "widget/form.html",
			"fieldGroups": (gridWidget.fieldGroups  || []).concat([{
				"id": 1,
				"name": "COLUMNS_FILTER",
				"opened": false,
				"collapsible": true,
				"label": ScreenService.i18n("More fields")
			}]),
			"fields": [],
			"actions": [
				{
					"id": 1,
					"name": "cancelAction",
					"label": ScreenService.i18n("Cancel"),
					"showAsAction": "back",
					"hideIcon": false,
					"isVisible": true,
					"events": [{
						"id": 1,
						"name": "ActionEvent",
						"code": "ScreenService.closePopup();"
					}]
				},
				{
					"id": 4,
					"name": "saveAndApplyFilterAction",
					"showAsAction": "never",
					"label": ScreenService.i18n("Save & Apply"),
					"isVisible": true,
					"events": [{
						"id": 1,
						"name": "ActionEvent",
						"code": "args.owner.widget.saveAndApplyFilter(args.owner.widget)"
					}]
				}
			]
		});

		return Util.extend(widget, functionsToInject);
	};

	this.createCurrentFormFilterWidget = function(gridWidget, functionsToInject) {
		var widget = metaDataFactory.widgetFactory({
			"id": "8514214420",
			"orderFieldsBySequenceInForm": true,
			"isVisible": true,
			"name": "filterPopup",
			"container": gridWidget.container,
			"label": ScreenService.i18n("Filter"),
			"template": "widget/form.html",
			"fieldGroups": (gridWidget.fieldGroups  || []).concat([{
				"id": 1,
				"name": "COLUMNS_FILTER",
				"opened": false,
				"collapsible": true,
				"label": ScreenService.i18n("More fields")
			}]),
			"events": Util.getNestedProperty(gridWidget, ['filterProperties', 'events']),
			"fields": [],
			"actions": [
				{
					"id": 1,
					"name": "cancelAction",
					"label": ScreenService.i18n("Back"),
					"showAsAction": "back",
					"hideIcon": false,
					"isVisible": true,
					"events": [{
						"id": 1,
						"name": "ActionEvent",
						"code": "ScreenService.closePopup();"
					}]
				},
				{
					"id": 4,
					"name": "saveAndApplyFilterAction",
					"showAsAction": "never",
					"label": ScreenService.i18n("Filter"),
					"isVisible": true,
					"events": [{
						"id": 1,
						"name": "ActionEvent",
						"code": "args.owner.widget.currentFilterClickPopup(args.owner.widget)"
					}]
				}
			]
		});
		return Util.extend(widget, functionsToInject);
	};

}

Configuration(function(ContextRegister) {
	ContextRegister.register("filterWidgetsCreator", FilterWidgetsCreator);
});