function loginController(LoginService, ScreenService, MessageService, eventAggregator, AuthService, GeneralMethods, templateManager) {

    var BEFORE_PROJECT_STARTUP = "beforeProjectStartup";
    var INVALID_TOKEN = "invalidToken";
    var LOGOUT = "logout";
    var saasURL = "https://mymeals.teknisa.com/backend/index.php";
    var saasIP = "mymeals.teknisa.com";

    this.login = function(widget) {
        if (getLocalVar("url")) {
            if (widget.isValid()) {
                LoginService.login(widget.currentRow)
                    .then(onLoginSuccess)
                    .catch(requestErrorCallback);
            }
        } else {
            MessageService.showMessageByCode("noServer", "alert");
        }
    };

    this.logout = function() {
        var params = {
            type: "CONFIRMATION",
            icon: "question",
            success: logout
        };
        MessageService.confirmMessageByCode(LOGOUT, null, params);
    };

    function logout() {
        LoginService.removeUserData();
        ScreenService.openWindow("login");
    }

    function onLoginSuccess(user) {
        buildUserAndHeader(user);
        if (user.session) {
            setLocalVar('Session', user.session);
        }
        ScreenService.openWindow("meals");
        GeneralMethods.user = user;

    }

    function buildUserAndHeader(user) {
        var userdesc = {
            NMFILIAL: user.cdfilial + ' - ' + user.nmfilial
        };
        ScreenService.buildUserData(user.nmoperador, [userdesc], userMethod);
    }

    window.buildUserAndHeader = buildUserAndHeader;

    function userMethod() {
        // @todo create user profile window
    }

    this.setUserInformation = function(widget) {
        widget.currentRow = {};
        widget.currentRow.IMAGE = templateManager.userData.image;
        widget.currentRow.NAME = templateManager.userData.name;
        widget.currentRow.NMFILIAL = templateManager.userData.descriptions[0] ? templateManager.userData.descriptions[0].NMFILIAL : "";
    };

    eventAggregator.subscribe(BEFORE_PROJECT_STARTUP, beforeProjectStartup);
    eventAggregator.onRequestError(onRequestError);
    eventAggregator.onRequestStart(onRequestStart);

    function onRequestStart(request) {
        request.requestConfig.headers.Session = getLocalVar("Session");
    }

    function beforeProjectStartup() {
        if (!getLocalVar("url")) {
            setLocalVar("url", saasURL);
            setLocalVar("ip", saasIP);
        }
        templateManager.updateURL(getLocalVar("url"));
        AuthService.validateSession().then(onLoginSuccess).catch(onValidateSessionError);
    }

    function onValidateSessionError() {
        ScreenService.openWindow("login");
        ScreenService.closePopup();
    }

    function onRequestError(response) {
        var error = response.data && response.data.data ? response.data.data : "";
        if (error && error.error && error.error.search(INVALID_TOKEN) !== -1) {
            error.error = INVALID_TOKEN;
            ScreenService.openWindow("login");
        }
    }

    this.configureServerPopup = function(widget) {
        var serverPopup = widget.container.getWidget("configServer");
        serverPopup.fields.forEach(function(field) {
            field.applyDefaultValue();
        });
        ScreenService.openPopup(serverPopup);
    };

    this.configureServer = function(widget) {
        if (widget.isValid()) {
            var row = widget.currentRow;
            var ip = row.ip;
            var path = row.path ? '/' + row.path : '';
            var port = !row.port ? '' : ':' + row.port;
            var url = 'http://' + ip + port + path + window.serviceUrl.replace('..', '');

            templateManager.updateURL(url);

            // do test request
            LoginService.testConnection().then(function(serverStatus) {
                // request success
                finishServerConfig(ip, row, url, serverStatus);
            }, function() {
                var url = 'https://' + ip + port + path + window.serviceUrl.replace('..', '');
                templateManager.updateURL(url);
                LoginService.testConnection().then(function(serverStatus) {
                    finishServerConfig(ip, row, url, serverStatus);
                }, function() {
                    // request fail
                    ScreenService.confirmMessage("Não foi possível conectar ao servidor, deseja manter o IP informado?", 'question', function() {
                        // yes
                        setLocalVar('ip', ip);
                        setLocalVar('port', row.port);
                        setLocalVar('path', row.path);
                        setLocalVar('url', url);
                        // close the ip popup
                        ScreenService.closePopup();
                        templateManager.updateTemplate();

                    }, function() {

                        // no, set last ip if exists
                        if (getLocalVar('url')) {
                            templateManager.updateURL(getLocalVar('url'));
                        }

                        widget.newRow();
                        templateManager.updateTemplate();

                    });
                });
            });
        }
    };

    function finishServerConfig(ip, row, url, serverStatus) {
        setLocalVar('ip', ip);
        setLocalVar('port', row.port);
        setLocalVar('path', row.path);
        setLocalVar('url', url);

        // close the ip popup
        ScreenService.closePopup();
        templateManager.updateTemplate();

        var conexaoValida = LoginService.compareVersion(serverStatus.dataset.conn.version);

        if (conexaoValida === "OK") {
            MessageService.successNotificationByCode("connectionSuccessful");
        } else {
            MessageService.showMessageByCode(conexaoValida);
        }
    }

    this.exitApplication = function() {
        if (RuiNativeInterface) {
            MessageService.confirmMessageByCode("exitAppQuestion").then(function() {
                RuiNativeInterface.closeApp();
            });
        }
    };
}

Configuration(function(ContextRegister) {
    ContextRegister.register("LoginController", loginController);
});
