function GeneralMethods(ScreenService, MessageService) {

    var timeoutErrorCode = "timeout";

    function requestErrorCallback(errorCode, variables, params) {
        if (!params) {
            params = {
                type: "error",
                icon: "error"
            };
        }
        MessageService.getMessageByCode(errorCode).then(function() {
            MessageService.showMessageByCode(errorCode, variables, params);
        }).catch(function() {
            ScreenService.showMessage(JSON.stringify(errorCode));
        });
    }

    function catchError(response) {
        throw response.data ? response.data.error : timeoutErrorCode;
    }

    window.catchError = catchError;
    window.requestErrorCallback = requestErrorCallback;

    var _user = {};

    Object.defineProperty(this, 'user', {
        set: function(obj) {
            _user = obj;
        },
        get: function() {
            return _user;
        }
    });

    this.createAndAppendField = function(widget, fieldProperties) {
        var field = {
            "id": moment().unix(),
            "isVisible": true,
            "showOnForm": true,
            "readOnly": true,
            "readOnlyOnEdit": false,
            "template": "rui-fields#field/text.html",
            "maxlength": 6,
            "class": "12",
            "events": [],
            "validations": {
                "required": true
            }
        };
        field = angular.extend({}, field, fieldProperties);
        metaDataFactory.factoryField(field, widget, true);
    };

    this.edit = function(widget) {
        widget.backupRow = angular.copy(widget.currentRow);
        widget.edit();
    };

    this.cancelEdit = function(widget) {
        for (var attr in widget.backupRow) {
            widget.currentRow[attr] = widget.backupRow[attr];
        }
        widget.fields.forEach(function(field) {
            field.readOnly = true;
        });
        widget.view();
        widget.currentRow.$error = {};
        widget.reload();
        widget.template = "widget/form.html";
    };

    this.valueToFloat = function(value) {
        if (typeof value === 'string' && !Util.isEmptyOrBlank(value)) {
            if (value.indexOf(',') === -1 && value.indexOf('.') === -1) {
                return parseFloat(value);
            } else if (value.indexOf(',') > value.indexOf('.')) {
                return parseFloat(value.replace(/\./g, '').replace(/\,/g, '.'));
            } else if (value.indexOf(',') < value.indexOf('.')) {
                return parseFloat(value.replace(/\,/g, ''));
            }
        } else {
            return value;
        }
    };
}

Configuration(function(ContextRegister) {
    ContextRegister.register('GeneralMethods', GeneralMethods);
});
